﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DesigningAndBuildingClasses
{
    /// <summary>
    /// Represents a color as red, green, blue, and alpha components.
    /// </summary>
    public class Color
    {
        /// <summary>
        /// Stores the red component of a color.
        /// </summary>
        private byte red;

        /// <summary>
        /// Stores the green component of a color.
        /// </summary>
        private byte green;

        /// <summary>
        /// Stores the blue component of a color.
        /// </summary>
        private byte blue;

        /// <summary>
        /// Stores the alpha component of a color.
        /// </summary>
        private byte alpha;

        /// <summary>
        /// Creates a new color with all of the components specified.
        /// </summary>
        public Color(byte red, byte green, byte blue, byte alpha)
        {
            this.red = red;
            this.green = green;
            this.blue = blue;
            this.alpha = alpha;
        }

        /// <summary>
        /// Creates a new color with only red, green, and blue components
        /// specified. In this case, alpha will be 255, or completely 
        /// opaque.
        /// </summary>
        public Color(byte red, byte green, byte blue)
        {
            this.red = red;
            this.green = green;
            this.blue = blue;
            this.alpha = 255;
        }

        /// <summary>
        /// Returns the red component of this color.
        /// </summary>
        /// <returns></returns>
        public byte GetRed()
        {
            return red;
        }

        /// <summary>
        /// Sets the red component of this color.
        /// </summary>
        /// <param name="red"></param>
        public void SetRed(byte red)
        {
            this.red = red;
        }

        /// <summary>
        /// Returns the green component of this color.
        /// </summary>
        /// <returns></returns>
        public byte GetGreen()
        {
            return green;
        }

        /// <summary>
        /// Sets the green component of this color.
        /// </summary>
        /// <param name="green"></param>
        public void SetGreen(byte green)
        {
            this.green = green;
        }

        /// <summary>
        /// Returns the blue component of this color.
        /// </summary>
        /// <returns></returns>
        public byte GetBlue()
        {
            return blue;
        }

        /// <summary>
        /// Sets the blue component of this color.
        /// </summary>
        /// <param name="blue"></param>
        public void SetBlue(byte blue)
        {
            this.blue = blue;
        }

        /// <summary>
        /// Returns the alpha component of this color.
        /// </summary>
        /// <returns></returns>
        public byte GetAlpha()
        {
            return alpha;
        }

        /// <summary>
        /// Sets the alpha component of this color.
        /// </summary>
        /// <param name="alpha"></param>
        public void SetAlpha(byte alpha)
        {
            this.alpha = alpha;
        }

        /// <summary>
        /// Returns the grayscale value of this color, which
        /// is the average of the red, green, and blue components.
        /// </summary>
        /// <returns></returns>
        public byte GetGrayscale()
        {
            return (byte)((red + green + blue) / 3);
        }
    }
}
